<?php
namespace Teknisa\COZ\Services;

use Teknisa\GER\Utilities\Util;
use Teknisa\GER\Utilities\Environment;
use Teknisa\COZ\Services\Pla08200RemoteRepository;
use Teknisa\COZ\Services\Pla10500RemoteRepository;

class Pla08200Service{
    protected $environment;
    protected $util;
    protected $pla08200remoteRepository;

    public function __construct($environment, $util, $pla08200remoteRepository, $pla10500remoteRepository) {
        $this->environment = $environment;
        $this->util = $util;
        $this->pla08200remoteRepository = $pla08200remoteRepository;
        $this->pla10500remoteRepository = $pla10500remoteRepository;
    }

    public function retornaFiliais($row) {
        if(empty($row['CDFILIAL'])){
            $fieldsToFilter = array("NRORG",
                                    "CDGRUPFILI",
                                    "CDCENTRAL");
            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
            $row['CDFILIAL'] = $this->pla08200remoteRepository->fetchFiliais($paramObjectArray);
        }
        return join('_', $row['CDFILIAL']);
    }

    public function retornaGrupServ($row) {
        if(empty($row['CDGRUPSERV'])) {   
            $row['CDGRUPSERV'] = $this->pla08200remoteRepository->fetchGrupserv();
        }
        return join('_', $row['CDGRUPSERV']);
    }

    public function retornaServico($row) {
        if(empty($row['FILISERV'])) {
            $row['CDGRUPSERV']   = $row['LISTAGRUPSERV'];
            $row['CDFILIAGREGA'] = $row['LISTAFILIAL'];

            $fieldsToFilter = array("NRORG",
                                    "CDGRUPFILI",
                                    "CDCENTRAL", 
                                    "CDGRUPSERV",
                                    "CDFILIAGREGA");
                                    
            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
            $row['FILISERV'] = $this->pla08200remoteRepository->fetchServicos($paramObjectArray);
        }
        return join('_', $row['FILISERV']);
    }

    public function concatArray($array, $key = false) {
        if ($key) {
            $string = '';
            foreach ($array as $item) {
              $string .= $item[$key].'_';
            }
            return trim($string, '_');
        }
        else return implode($array, "_");
    }

    public function retornaDataFinal($row) {
        $fieldsToFilter = array("DTCARDAFIN");

        $dtfinal = array(array("columnName" => "DTCARDAFIN",
                                "operator"  => "=",
                                "value"     => $row[0]["DTCARDAFIN"]));
                                
        $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
        $dtcardafin = $this->pla10500remoteRepository->fetchDataFinal($dtfinal);
        return $dtcardafin[0];
    }

    /*
     * Constroi um array de arrays a partir de uma row, no formato chave=>valor,
     * e um array de strings especificando quais das colunas deverão ser usadas
     * pega todos os valores de $row com chaves presentes em $filterKeys
     * o array_flip faz com que os valores de $filterKeys sejam chaves
     */
    public function buildFilterArrayFromRow($row, $filterKeys) {
        $paramValueArray = array_intersect_key($row, array_flip($filterKeys));
        
        return array_map(function($key, $value) {
            return array("columnName" => $key,
                         "operator"   => "=",
                         "value"      => $value); 
        }, array_keys($paramValueArray), $paramValueArray);
    }
}